# 
#  R script for generating a plot with mBAF data
#  The plot is divided into two parts:
#  1) Average mBAF per region for a sample. Regions that differ too much (pval < 0.01) 
#     compared to the normal reference samples are highlighted
#  2) Distribution of average mBAF values compared to a set of reference normal samples
#
#  Parameters
#  sampleName: Name of the sample (alignment)
#  mBafFile: Path to tab-separated file with mBAF data. The file should have the following
#            columns: Region, Count, AvgMBaf, PVal
#  datadir: Directory with reference data for the normal samples
#  plotFile: Name of file to create
#  width, height: Size of the plot
#  pval: p-value limit for regions that should be highlighted
#  minCount: If the number of SNPs in a region is below this value that bar is not colored or highlighted
#
mBafPlot = function(sampleName, mBafFile="mbafdata.txt", datadir=".", plotFile="mbafplot.png", width=1600, height=1000, pval=0.01, minCount=20)
{
	# Load mBAF data
	# Tab-separated with headers: Region, Count, AvgMBaf, PVal
	mbafdata <- read.table(mBafFile, header=T, sep="\t", check.names=F, stringsAsFactors=F);
	
	# Load mBAF data for normal references
	# Tab-separated with headers: Sample, 1p, 1q, ...
	normalrefs <- read.table(paste(datadir, "normal-refs.txt", sep="/"), header=T, sep="\t", check.names=F, stringsAsFactors=F);
	# We only need the mBAF values as one long list of values
	normalrefs <- na.omit(unlist(normalrefs[,2:length(names(normalrefs))], use.names=F));
	
	png(plotFile, width=width, height=height);
	
	# Prepare for plottting 2 plots in a single window above each other
	# and make room for adding title
	par(mfrow=c(2, 1), oma=c(0,0,2,0), mar=c(4.5, 4.5, 2.0, 2.5));
	
	# *** Plot 1: Bar plot with Region on X axis and average mBAF on Y axis
	# Regions with low p-value are highlighted (but only if we have enough SNPs)
	barcolors <- barcolors <- ifelse(mbafdata$Count < minCount, "white", ifelse(mbafdata$PVal < pval, "darkblue", "gray"));
	barborders <- ifelse(mbafdata$Count < minCount, "gray", "black");
	bp <- barplot(mbafdata$AvgMBaf, main="Average mBAF per region", cex.main=1.25, font.main=1, ylab="mBAF", ylim=c(0.5,0.9), names.arg=mbafdata$Region, col=barcolors, border=barborders, xpd=F, xaxs = "i", xlim=c(0, 1.2*length(mbafdata$AvgMBaf)));
	# Add Count (of SNPS making up the average) above each bar
	text(x=bp, y=ifelse(is.na(mbafdata$AvgMBaf), 0.5, mbafdata$AvgMBaf), label=mbafdata$Count, cex=0.85, pos=3);
	# Draw x-axis at y=0.5
	axis(1, pos=0.5, at=c(0,1.2*length(mbafdata$AvgMBaf)), tick=T, labels=F);
	
	# *** Plot 2: Density plot of the average mBAF values
	densitySample = density(na.omit(mbafdata$AvgMBaf));
	densityNormal = density(normalrefs);
	maxY <- max(densitySample$y, densityNormal$y);
	plot(densitySample, main="Distribution of mBAF in sample vs. normals", cex.main=1.25, font.main=1, xlab="mBAF", ylab="", yaxt="n", bty="n", xaxs = "i", xlim=c(0.5, 0.9), ylim=c(0, maxY), lwd=2);
	# Fill sample distribution with semi-transparant gray
	polygon(densitySample, col=adjustcolor("lightgray", alpha.f=0.5));
	# Draw normal reference distribution with dashed line
	lines(densityNormal, col="red", lty=2, lwd=2);
	# Mark individual mBAF values with small ticks on the x-axis
	rug(mbafdata$AvgMBaf)
	
	# Add title
	title(main=sampleName, outer=T, cex.main=1.75);
	off <- dev.off();
}
