#
# Run multiple 'models' for a single 'gene.tsv' file
# Output is one result per line in the same order as the 'models' paramter.
#
# tsvFile: Path to gene.tsv file from StringTie
# sourceDir: Path to directory with the R files that are needed (see loadSources())
# modelsDir: Path to the directory with model RData files
# models: File names of RData files, each file represents a single SSP model
#
runSSP = function(tsvFile, sourceDir, modelsDir, models)
{
	
	loadLibraries();
	loadSources(sourceDir);
	
	# load gene annotations
	load(paste(sourceDir, "Gene.ID.ann.Rdata", sep="/"));
	
	# read tsv
	mymatrix <- read_StringTie_tsv_FPKM(tsvFile, id=Gene.ID.ann$Gene.ID);
	
	# translate gene id
	myid <- translate_id2entrez(id=rownames(mymatrix), ann=Gene.ID.ann, e=TRUE);
	
	# load ssp model
	for (model in models)
	{
		load(paste(modelsDir, model, sep="/"));	# loaded ssp object must be named aims.gs 
	
		aims.gs <- fixTheIsNumericProblem(aims.gs);
		
		# apply ssp model
		resultslist <- applyAIMS(mymatrix, myid, aims.gs);
		all.probs <- c(resultslist$all.probs[[1]])
		names(all.probs) <- colnames(resultslist$all.probs[[1]])
		
		# print result to stdout (semicolon-separated list):
		# result class;class1=score;class2=score;...;classN=score
		cat("<", resultslist$cl[[1]], sep="");
		for (n in names(all.probs))
		{
			cat("; ", n, "=", all.probs[n], sep="");
		}
		cat(">\n");
	}
}

# We need to add the "isnumeric" variable to the "aims.gs$one.vs.all.tsp" object 
# and set the value to TRUE for named "tables", otherwise we will get a failure: 
#   argument is of length zero
# inside e1071 package 'predict.naiveBayes' function 
# on a line with code:
#   if (object$isnumeric[i] != is.numeric(newdata[[i]]))
fixTheIsNumericProblem = function(aims.gs)
{
	k <- aims.gs$k;
	tmp <- aims.gs$one.vs.all.tsp[[k]];
	aims.gs$one.vs.all.tsp[[k]]$isnumeric[names(tmp$tables)] <- TRUE;
	return (aims.gs);
}


loadSources = function(sourceDir)
{
	# load SSP_functions
	source(paste(sourceDir, "SSP_functions.R", sep="/"))
	
	# load functions for reading tsv and for translating id to entrez
	source(paste(sourceDir, "read_StringTie_tsv_FPKM_v1.R", sep="/"))
	source(paste(sourceDir, "translate_id2entrez_v1.R", sep="/"))
}

loadLibraries = function()
{
	library(caret);
	library(gridExtra);
	library(grid);
	library(ggplot2);
	library(reshape2);
	require(e1071);
}